@@ -1,11 +1,47 @@
-"""
-  Copyright (c) 2000 Gregory Trubetskoy.  All rights reserved.
- 
-  This file is part of mod_python. See COPYRIGHT file for details.
-
-  $Id: apache.py,v 1.22 2000/10/21 20:24:54 gtrubetskoy Exp $
-
-"""
+ # Copyright (c) 2000 Gregory Trubetskoy.  All rights reserved.
+ #
+ # Redistribution and use in source and binary forms, with or without
+ # modification, are permitted provided that the following conditions
+ # are met:
+ #
+ # 1. Redistributions of source code must retain the above copyright
+ #    notice, this list of conditions and the following disclaimer. 
+ #
+ # 2. Redistributions in binary form must reproduce the above copyright
+ #    notice, this list of conditions and the following disclaimer in
+ #    the documentation and/or other materials provided with the
+ #    distribution.
+ #
+ # 3. The end-user documentation included with the redistribution, if
+ #    any, must include the following acknowledgment: "This product 
+ #    includes software developed by Gregory Trubetskoy."
+ #    Alternately, this acknowledgment may appear in the software itself, 
+ #    if and wherever such third-party acknowledgments normally appear.
+ #
+ # 4. The names "mod_python", "modpython" or "Gregory Trubetskoy" must not 
+ #    be used to endorse or promote products derived from this software 
+ #    without prior written permission. For written permission, please 
+ #    contact grisha@modpython.org.
+ #
+ # 5. Products derived from this software may not be called "mod_python"
+ #    or "modpython", nor may "mod_python" or "modpython" appear in their 
+ #    names without prior written permission of Gregory Trubetskoy.
+ #
+ # THIS SOFTWARE IS PROVIDED BY GREGORY TRUBETSKOY ``AS IS'' AND ANY
+ # EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ # PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GREGORY TRUBETSKOY OR
+ # HIS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ # SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ # NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ # HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+ # STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+ # OF THE POSSIBILITY OF SUCH DAMAGE.
+ # ====================================================================
+ #
+ # $Id: apache.py,v 1.28 2000/12/06 03:05:37 gtrubetskoy Exp $
 
 import sys
 import string
@@ -18,8 +54,6 @@ import imp
 import types
 import _apache
 
-# XXX consider using intern() for some strings
-
 # a small hack to improve PythonPath performance. This
 # variable stores the last PythonPath in raw (unevaled) form.
 _path = None
@@ -39,10 +73,7 @@ class Request:
         try:
             return getattr(self._req, attr)
         except AttributeError:
-            try:
-                return self.__dict__[attr]
-            except KeyError:
-                raise AttributeError, attr
+            raise AttributeError, attr
 
     def __setattr__(self, attr, val):
         try:
@@ -64,40 +95,6 @@ class CallBack:
     def __init__(self):
         self.req = None
 
-
-    def resolve_object(self, module, object_str):
-        """
-        This function traverses the objects separated by .
-        (period) to find the last one we're looking for:
-
-           From left to right, find objects, if it is
-           an unbound method of a class, instantiate the
-           class passing the request as single argument
-        """
-
-        obj = module
-        
-        for obj_str in  string.split(object_str, '.'):
-
-            parent = obj
-
-            # this adds a little clarity if we have an attriute error
-            if obj == module and not hasattr(module, obj_str):
-                if hasattr(module, "__file__"):
-                    s = "module '%s' contains no '%s'" % (module.__file__, obj_str)
-                    raise AttributeError, s
-
-            obj = getattr(obj, obj_str)
-            
-            if hasattr(obj, "im_self") and not obj.im_self:
-                # this is an unbound method, its class
-                # needs to be instantiated
-                instance = parent(self.req)
-                obj = getattr(instance, obj_str)
-                
-        return obj
-
-
     class HStack:
         """
         The actual stack string lives in the request object so
@@ -108,6 +105,7 @@ class CallBack:
             self.req = req
 
         def pop(self):
+
             handlers = string.split(self.req.hstack)
             if not handlers:
                 return None
@@ -140,7 +138,7 @@ class CallBack:
             dirs = _req.get_all_dirs()
 
             hstack = self.HStack(_req)
-        
+
             handler = hstack.pop()
             while handler:
 
@@ -167,7 +165,10 @@ class CallBack:
                         if sys.path != newpath:
                             sys.path[:] = newpath
                 else:
-                    dir = _req.get_all_dirs()[htype]
+                    if config.has_key("PythonHandlerModule"):
+                        dir = _req.get_all_dirs()["PythonHandlerModule"]
+                    else:
+                        dir = _req.get_all_dirs()[htype]
                     if dir not in sys.path:
                         sys.path[:0] = [dir]
 
@@ -175,32 +176,44 @@ class CallBack:
                 module = import_module(module_name, _req)
 
                 # find the object
-                object = self.resolve_object(module, object_str)
+                silent = config.has_key("PythonHandlerModule")
+                object = resolve_object(self.req, module, object_str, silent)
 
-                # call the object
-                if config.has_key("PythonEnablePdb"):
-                    if config["PythonEnablePdb"]:
-                        result = pdb.runcall(object, self.req)
-                else:
-                    result = object(self.req)
+                if object:
 
-                # stop cycling through handlers
-                if result != OK:
-                    break
-                
+                    # call the object
+                    if config.has_key("PythonEnablePdb"):
+                        if config["PythonEnablePdb"]:
+                            result = pdb.runcall(object, self.req)
+                    else:
+                        result = object(self.req)
+
+                    # stop cycling through handlers
+                    if result != OK:
+                        break
+                    
+                elif silent:
+                    result = DECLINED
+                        
                 handler = hstack.pop()
 
-
         except SERVER_RETURN, value:
             # SERVER_RETURN indicates a non-local abort from below
             # with value as (result, status) or (result, None) or result
             try:
-                if type(value) == type(()):
-                    (result, status) = value
+                if len(value.args) == 2:
+                    (result, status) = value.args
                     if status:
                         _req.status = status
                 else:
-                    result, status = value, value
+                    result = value.args[0]
+                    
+                if type(result) != type(7):
+                    s = "Value raised with SERVER_RETURN is invalid. It is a "
+                    s = s + "%s, but it must be a tuple or an int." % type(result)
+                    _apache.log_error(s, APLOG_NOERRNO|APLOG_ERR, _req.server)
+                    return HTTP_INTERNAL_SERVER_ERROR
+
             except:
                 pass
 
@@ -269,7 +282,7 @@ class CallBack:
             etb = None
             return DONE
 
-def import_module(module_name, req=None):
+def import_module(module_name, req=None, path=None):
     """ 
     Get the module to handle the request. If
     autoreload is on, then the module will be reloaded
@@ -313,10 +326,21 @@ def import_module(module_name, req=None):
         # import the module for the first time
         else:
 
-            module = __import__(module_name)
-            components = string.split(module_name, '.')
-            for cmp in components[1:]:
-                module = getattr(module, cmp)
+            parts = string.split(module_name, '.')
+            for i in range(len(parts)):
+                f, p, d = imp.find_module(parts[i], path)
+                try:
+                    mname = string.join(parts[:i+1], ".")
+                    module = imp.load_module(mname, f, p, d)
+                finally:
+                    if f: f.close()
+                if hasattr(module, "__path__"):
+                    path = module.__path__
+
+##            module = __import__(module_name)
+##            components = string.split(module_name, '.')
+##            for cmp in components[1:]:
+##                module = getattr(module, cmp)
 
         # find out the last modification time
         # but only if there is a __file__ attr
@@ -345,6 +369,41 @@ def import_module(module_name, req=None):
 
     return module
 
+def resolve_object(req, module, object_str, silent=0):
+    """
+    This function traverses the objects separated by .
+    (period) to find the last one we're looking for:
+
+       From left to right, find objects, if it is
+       an unbound method of a class, instantiate the
+       class passing the request as single argument
+    """
+
+    obj = module
+
+    for obj_str in  string.split(object_str, '.'):
+
+        parent = obj
+
+        # don't throw attribute errors when silent
+        if silent and not hasattr(module, obj_str):
+            return None
+
+        # this adds a little clarity if we have an attriute error
+        if obj == module and not hasattr(module, obj_str):
+            if hasattr(module, "__file__"):
+                s = "module '%s' contains no '%s'" % (module.__file__, obj_str)
+                raise AttributeError, s
+
+        obj = getattr(obj, obj_str)
+
+        if hasattr(obj, "im_self") and not obj.im_self:
+            # this is an unbound method, its class
+            # needs to be instantiated
+            instance = parent(req)
+            obj = getattr(instance, obj_str)
+
+    return obj
 
 def build_cgi_env(req):
     """
@@ -409,7 +468,12 @@ class CGIStdin(NullIO):
             self.buf = ""
             return result
         else:
-            s = self.req.read(n)
+            if self.buf:
+                s = self.buf[:n]
+                n = n - len(s)
+            else:
+                s = ""
+            s = s + self.req.read(n)
             self.pos = self.pos + len(s)
             return s
 
@@ -441,6 +505,7 @@ class CGIStdin(NullIO):
         # carve out the piece, then shorten the buffer
         result = self.buf[:i+1]
         self.buf = self.buf[i+1:]
+        self.pos = self.pos + len(result)
         return result
         
 
@@ -552,6 +617,8 @@ def init():
 ## Some functions made public
 make_table = _apache.make_table
 log_error = _apache.log_error
+parse_qs = _apache.parse_qs
+parse_qsl = _apache.parse_qsl
 
 ## Some constants
 
@@ -641,7 +708,7 @@ REMOTE_DOUBLE_REV = 3
 
 REQ_ABORTED = HTTP_INTERNAL_SERVER_ERROR
 REQ_EXIT = "REQ_EXIT"         
-SERVER_RETURN = "SERVER_RETURN"
+SERVER_RETURN = _apache.SERVER_RETURN
 PROG_TRACEBACK = "PROG_TRACEBACK"
 
 
