@@ -1,4 +1,4 @@
-/*====================================================================
+/* ====================================================================
  * Copyright (c) 2000 Gregory Trubetskoy.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -13,27 +13,20 @@
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by Gregory Trubetskoy
- *    for use in the mod_python module for Apache HTTP server 
- *    (http://www.modpython.org/)."
+ * 3. The end-user documentation included with the redistribution, if
+ *    any, must include the following acknowledgment: "This product 
+ *    includes software developed by Gregory Trubetskoy."
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear.
  *
  * 4. The names "mod_python", "modpython" or "Gregory Trubetskoy" must not 
  *    be used to endorse or promote products derived from this software 
  *    without prior written permission. For written permission, please 
- *    contact grisha@ispol.com.
+ *    contact grisha@modpython.org.
  *
  * 5. Products derived from this software may not be called "mod_python"
  *    or "modpython", nor may "mod_python" or "modpython" appear in their 
- *    names without prior written permission of Gregory Trubetskoy. For 
- *    written permission, please contact grisha@ispol.com..
- *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by Gregory Trubetskoy
- *    for use in the mod_python module for Apache HTTP server 
- *    (http://www.modpython.org/)."
+ *    names without prior written permission of Gregory Trubetskoy.
  *
  * THIS SOFTWARE IS PROVIDED BY GREGORY TRUBETSKOY ``AS IS'' AND ANY
  * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -51,7 +44,7 @@
  *
  * connobject.c 
  *
- * $Id: connobject.c,v 1.1 2000/10/16 20:58:57 gtrubetskoy Exp $
+ * $Id: connobject.c,v 1.6 2000/12/18 19:50:03 gtrubetskoy Exp $
  *
  */
 
@@ -102,7 +95,7 @@ static struct memberlist conn_memberlist[] = {
     {"remote_ip",          T_STRING,    OFF(remote_ip),          RO},
     {"remote_host",        T_STRING,    OFF(remote_host),        RO},
     {"remote_logname",     T_STRING,    OFF(remote_logname),     RO},
-    {"user",               T_STRING,    OFF(user),               RO},
+    {"user",               T_STRING,    OFF(user),                 },
     {"ap_auth_type",       T_STRING,    OFF(ap_auth_type),       RO},
     /* XXX aborted, keepalive, keptalive, double_reverse ? */
     {"local_ip",           T_STRING,    OFF(remote_ip),          RO},
@@ -215,6 +208,30 @@ static PyObject * conn_getattr(connobject *self, char *name)
 
 }
 
+/**
+ ** conn_setattr
+ **
+ *  Set connection object attribute
+ */
+
+static int conn_setattr(connobject *self, char* name, PyObject* value)
+{
+
+    if (value == NULL) {
+	PyErr_SetString(PyExc_AttributeError,
+			"can't delete connection attributes");
+	return -1;
+    }
+    else if (strcmp(name, "user") == 0) {
+	self->conn->user = 
+	    ap_pstrdup(self->conn->pool, PyString_AsString(value));
+	return 0;
+    }
+    else
+	return PyMember_Set((char *)self->conn, conn_memberlist, name, value);
+
+}
+
 PyTypeObject MpConn_Type = {
     PyObject_HEAD_INIT(NULL)
     0,
@@ -224,7 +241,7 @@ PyTypeObject MpConn_Type = {
     (destructor) conn_dealloc,       /*tp_dealloc*/
     0,                               /*tp_print*/
     (getattrfunc) conn_getattr,      /*tp_getattr*/
-    0,                               /*tp_setattr*/
+    (setattrfunc) conn_setattr,      /*tp_setattr*/
     0,                               /*tp_compare*/
     0,                               /*tp_repr*/
     0,                               /*tp_as_number*/
